
From: James Bottomley <James.Bottomley@SteelEye.com>

The alder has an intel Extended Express System Support Controller which
presents apparently spurious BARs.  When the pci resource code tries to
reassign these BARs, the second IO-APIC gets disabled (with disastrous
consequences).

The first BAR is the actual IO-APIC, the remaining five bars seem to be
spurious resources, so we forcibly insert the first one into the resource
tree and clear all the others.



---

 25-akpm/drivers/pci/quirks.c    |   24 ++++++++++++++++++++++++
 25-akpm/include/linux/pci_ids.h |    1 +
 2 files changed, 25 insertions(+)

diff -puN drivers/pci/quirks.c~alder-io_apic-quirk drivers/pci/quirks.c
--- 25/drivers/pci/quirks.c~alder-io_apic-quirk	Fri Feb 20 16:14:00 2004
+++ 25-akpm/drivers/pci/quirks.c	Fri Feb 20 16:14:00 2004
@@ -789,6 +789,29 @@ static void __init quirk_sis_96x_compati
 	sis_96x_compatible = 1;
 }
 
+#ifdef CONFIG_X86_IO_APIC
+static void __init quirk_alder_ioapic(struct pci_dev *pdev)
+{
+	int i;
+
+	if ((pdev->class >> 8) != 0xff00)
+		return;
+
+	/* the first BAR is the location of the IO APIC...we must
+	 * not touch this (and it's already covered by the fixmap), so
+	 * forcibly insert it into the resource tree */
+	if(pci_resource_start(pdev, 0) && pci_resource_len(pdev, 0))
+		insert_resource(&iomem_resource, &pdev->resource[0]);
+
+	/* The next five BARs all seem to be rubbish, so just clean
+	 * them out */
+	for(i=1; i < 6; i++) {
+		memset(&pdev->resource[i], 0, sizeof(pdev->resource[i]));
+	}
+
+}
+#endif
+
 #ifdef CONFIG_SCSI_SATA
 static void __init quirk_intel_ide_combined(struct pci_dev *pdev)
 {
@@ -914,6 +937,7 @@ static struct pci_fixup pci_fixups[] __d
 	{ PCI_FIXUP_FINAL,	PCI_VENDOR_ID_SI,	PCI_ANY_ID,			quirk_ioapic_rmw },
         { PCI_FIXUP_FINAL,      PCI_VENDOR_ID_AMD, PCI_DEVICE_ID_AMD_8131_APIC,
           quirk_amd_8131_ioapic }, 
+	{ PCI_FIXUP_HEADER,	PCI_VENDOR_ID_INTEL,	PCI_DEVICE_ID_INTEL_EESSC,	quirk_alder_ioapic },
 #endif
 	{ PCI_FIXUP_HEADER,	PCI_VENDOR_ID_VIA,	PCI_DEVICE_ID_VIA_82C586_3,	quirk_via_acpi },
 	{ PCI_FIXUP_HEADER,	PCI_VENDOR_ID_VIA,	PCI_DEVICE_ID_VIA_82C686_4,	quirk_via_acpi },
diff -puN include/linux/pci_ids.h~alder-io_apic-quirk include/linux/pci_ids.h
--- 25/include/linux/pci_ids.h~alder-io_apic-quirk	Fri Feb 20 16:14:00 2004
+++ 25-akpm/include/linux/pci_ids.h	Fri Feb 20 16:14:00 2004
@@ -1928,6 +1928,7 @@
 #define PCI_DEVICE_ID_GENROCO_HFP832	0x0003
 
 #define PCI_VENDOR_ID_INTEL		0x8086
+#define PCI_DEVICE_ID_INTEL_EESSC	0x0008
 #define PCI_DEVICE_ID_INTEL_21145	0x0039
 #define PCI_DEVICE_ID_INTEL_82375	0x0482
 #define PCI_DEVICE_ID_INTEL_82424	0x0483

_
